-- Import other files variables
local _, L = ...;

local POWEREDSKILLS = {
  LASTBUFFLIST = {},
  SKILLLIST = {},
  SETTINGS = {
    debug = false,
  },
};

-- SETTINGS
POWEREDSKILLS.LocalizedClassName, POWEREDSKILLS.PlayerClassName, POWEREDSKILLS.ClassIndex = UnitClass("Player");

if (POWEREDSKILLS.PlayerClassName == "WARRIOR") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_WARRIOR;

elseif (POWEREDSKILLS.PlayerClassName == "PALADIN") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_PALADIN;

elseif (POWEREDSKILLS.PlayerClassName == "HUNTER") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_HUNTER;

elseif (POWEREDSKILLS.PlayerClassName == "ROGUE") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_ROGUE;

elseif (POWEREDSKILLS.PlayerClassName == "PRIEST") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_PRIEST;

elseif (POWEREDSKILLS.PlayerClassName == "DEATHKNIGHT") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_DEATHKNIGHT;

elseif (POWEREDSKILLS.PlayerClassName == "SHAMAN") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_SHAMAN;

elseif (POWEREDSKILLS.PlayerClassName == "MAGE") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_MAGE;

elseif (POWEREDSKILLS.PlayerClassName == "WARLOCK") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_WARLOCK;

elseif (POWEREDSKILLS.PlayerClassName == "MONK") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_MONK;

elseif (POWEREDSKILLS.PlayerClassName == "DRUID") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_DRUID;

elseif (POWEREDSKILLS.PlayerClassName == "DEMONHUNTER") then
  POWEREDSKILLS.SKILLLIST = PoweredSkills_DEMONHUNTER;
end


function log(param)
  if (POWEREDSKILLS.SETTINGS.debug) then
    print(param);
  end
end

-- Register event changes
function PoweredSkills_RegisterEvents()
  PoweredSkills:RegisterEvent('UNIT_AURA', 'Player');

  PoweredSkills:SetScript("OnEvent",
    function(self, event, ...)
      if (event == 'UNIT_AURA') then

        -- Save buffs in a table
        local buffs = {};
        local buffIndex = 1;
        local buff;

        while buffIndex do

          -- Get buff
          name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff("player", buffIndex);

          -- If there is a buff
          if (spellId) then
            buff = {
              ["spellId"] = spellId,
              ["duration"] = expirationTime,
            };

            -- Stores the buff on the array
            buffs[buffIndex] = buff;

            -- Update index
            buffIndex = buffIndex + 1;

          -- If there is no buff
          else
            buffIndex = false
          end
        end;

        PoweredSkills_CheckFinishedBuffs(buffs);
        PoweredSkills_CheckCurrentBuffs(buffs);
      end
    end
  )
end

-- Check for buffs that have finished
function PoweredSkills_CheckFinishedBuffs(buffList)
  -- Check all previous buffs
  for y=1, table.getn(POWEREDSKILLS.LASTBUFFLIST), 1 do
    local lastBuffId = POWEREDSKILLS.LASTBUFFLIST[y]["spellId"];
    local isPresent = false;

    -- For each current buff
    for x=1, table.getn(buffList), 1 do
      local buffId = buffList[x]["spellId"];
      -- If they match, buff is still present
      if (lastBuffId == buffId) then
        isPresent = true;
      end
    end

    -- If the buff is not pressent
    if (isPresent == false) then

      -- Loop the Powered Skills table
      for x=1, table.getn(POWEREDSKILLS.SKILLLIST), 1 do
        local poweredSkill = POWEREDSKILLS.SKILLLIST[x];

        -- If this was an empoweringBuff
        if (lastBuffId == poweredSkill["empoweringBuff"]) then

          -- Set up as inactive
          PoweredSkills_LosePoweringBuff(poweredSkill);
        end
      end
    end
  end

  POWEREDSKILLS.LASTBUFFLIST = buffList;
end

-- When a powering buff is over
function PoweredSkills_LosePoweringBuff(poweredSkill)
  local skillButtonName, skillButton;

  poweredSkill["powered"] = false;

  -- Set button name
  skillButtonName = "PoweredSkills_Skill_" .. poweredSkill["spellId"];

  -- Get the button
  skillButton = _G[skillButtonName];

  -- Restart text
  skillButton:SetText(0);

  -- Hide the button
  skillButton:Hide();

  -- Updates layout
  PoweredSkills_Render();
end

-- Check for new buffs
function PoweredSkills_CheckCurrentBuffs(buffList)
  -- Loop the entire table
  for x=1, table.getn(buffList), 1 do
    for y=1, table.getn(POWEREDSKILLS.SKILLLIST), 1 do

      -- If the ID is 'Oportunidad' id
      local buffId = buffList[x]["spellId"];
      local duration = buffList[x]["duration"];
      local poweredSkill = POWEREDSKILLS.SKILLLIST[y];

      if (buffId == poweredSkill["empoweringBuff"]) then
        PoweredSkills_GetPoweringBuff(poweredSkill, duration);
      end
    end
  end
end

-- When we get a powering buff
function PoweredSkills_GetPoweringBuff(poweredSkill, duration)
  local skillButtonName, skillButton, skillIcon;
  local firstPoint, relativeTo, secondPoint, marginLeft, marginTop;
  local sname, srank, sicon, scastingTime, sminRange, smaxRange, sspellID = GetSpellInfo(poweredSkill["spellId"]);

  -- Stores current duration of the buff
  poweredSkill["duration"] = duration;

  -- If not active, render it
  if (poweredSkill["powered"] == false) then
    -- Mark skill as active
    poweredSkill["powered"] = true;

    -- Create a button
    skillButtonName = "PoweredSkills_Skill_" .. poweredSkill["spellId"];

    -- Reuse or create button
    if (_G[skillButtonName]) then

      -- Select existing button
      skillButton = _G[skillButtonName];
    else

      -- Create new button
      skillButton = CreateFrame("Button", skillButtonName, PoweredSkills, "PopupButtonTemplate");
      -- skillButton = CreateFrame("Button", skillButtonName, PoweredSkills, "SecureActionButtonTemplate");

      -- Set attributes to make it clickable
      -- skillButton:SetAttribute("type", "spell");
      -- skillButton:SetAttribute("spell", sname);
      -- skillButton:SetAttribute("target", "target")

      -- Save button order
      skillButton:SetID(poweredSkill["spellId"]);

      -- Set size for the button and parent
      skillButton:SetSize(48, 48);

      -- Get and set icon
      skillIcon = GetSpellTexture(poweredSkill["spellId"]);
      skillButton.icon = skillButton:CreateTexture(skillButtonName .. "_Icon", "ARTWORK");
      skillButton.icon:SetWidth(38);
      skillButton.icon:SetHeight(38);
      skillButton.icon:SetPoint("TOPLEFT", 5, -6);
      skillButton.icon:SetTexture(skillIcon);

      -- Backdrop
      skillButton.backdrop = skillButton:CreateTexture(skillButtonName .. "_Backdrop", "ARTWORK");
      skillButton.backdrop:SetWidth(38);
      skillButton.backdrop:SetHeight(38);
      skillButton.backdrop:SetPoint("TOPLEFT", 5, -6);
      skillButton.backdrop:SetColorTexture(0, 0, 0, 0.4);

      -- Set a textframe
      skillButtonTextName = "PoweredSkills_Skill_" .. poweredSkill["spellId"] .. "_Text";
      skillButton.text = skillButton:CreateFontString(skillButtonName .. "_Text", "OVERLAY", "GameFontNormalLarge");
      local fontName, fontSize, fontFlags = skillButton.text:GetFont();
      skillButton.text:SetFont(fontName, 24, fontFlags);
      skillButton.text:SetWidth(38);
      skillButton.text:SetHeight(38);
      skillButton.text:SetPoint("CENTER", 0, -3);
      skillButton.text:SetTextColor(1, 1, 1);
    end

    -- Update empowering duration
    PoweredSkills_UpdateRemainingTime(skillButton, poweredSkill);

    -- Show button
    skillButton:Show();

    -- Sound an Alert
    PlaySoundFile("Interface\\AddOns\\PoweredSkills\\sounds\\Hey.ogg");
    PlaySoundFile("Interface\\AddOns\\PoweredSkills\\sounds\\Hey.ogg");
  end

  -- Updates layout
  PoweredSkills_Render();
end

-- Recursive function to update expiration time
function PoweredSkills_UpdateRemainingTime(skillButton, poweredSkill)

  -- Calc remainig time
  local duration = math.floor(poweredSkill["duration"] - GetTime());

  -- Set duration number on the button
  skillButton.text:SetText(duration);

  -- If duration is bigger than 0 and skill is powered
  if (duration > 0 and poweredSkill["powered"] == true) then

    -- Wait for 1 second
    C_Timer.After(0.5, function()

      -- Update text again
      PoweredSkills_UpdateRemainingTime(skillButton, poweredSkill);
    end);

  -- If skill is not powered anymore
  else
    poweredSkill["duration"] = false;
  end
end

-- Reorder buttons into the layout
function PoweredSkills_Render()
  local kids = { PoweredSkills:GetChildren() };
  local visibleCounter = 0;

  -- Go through each children
  for x, child in ipairs(kids) do

    -- If it's not hidden (active buff)
    if (child:IsShown()) then

      -- Calc new position for the elements
      firstPoint = "TOPLEFT";
      relativeTo = PoweredSkills;
      secondPoint = "TOPLEFT";
      marginLeft = 0 + (48 * visibleCounter);
      marginTop = 0;

      -- Set it up
      child:SetPoint(firstPoint, relativeTo, secondPoint, marginLeft, marginTop);

      -- Updates visible counter
      visibleCounter = visibleCounter + 1;
    end
  end

  -- If no buttons, hide the interface
  if (visibleCounter == 0) then
    PoweredSkills:Hide();

  -- If there are buttons
  else

    -- Reorganize buttons
    PoweredSkills:SetSize((48 * visibleCounter), 48);

    -- Show the interface
    PoweredSkills:Show();
  end
end

-- On load
function PoweredSkills_OnLoad(self)
  if ( PoweredSkills ) then
    -- Register things/events
    PoweredSkills_RegisterEvents();
  end
end
